import java.io.*;
import java.util.*;
import java.text.*;

public class CNVQualityMerge {
    public static void main(String[] args) {
	CNVQualityMerge cnvq = new CNVQualityMerge();
	cnvq.process(args);
    }

    public void process (String[] args) {
	String segFile = null;
	String CRESTFile = "CREST_final_report.txt";
	String CRESTInput = null;
	String aiFile = null;
	double lpower = 3, gpower = 3, SV_minVal = 0.4, thresh_merging = 0.3, thresh_diff = 0.125, thresh_log = 0.170;
	int window = 100;
	boolean noSV = false;
	SV[][] svs = new SV[25][10];
	int[] counts = new int[25];
	boolean male = false, readAI = false;
	int[][] snps = new int[23][];
	    int ii = 0;
	    while (ii < args.length) {
		if (args[ii].compareToIgnoreCase("-seg") == 0) {// Segmentation file
		    ii++;
		    segFile = args[ii++];
		} else if (args[ii].compareToIgnoreCase("-c1") == 0) {// Orignial CREST file
		    ii++;
		    CRESTInput = args[ii++];
		} else if (args[ii].compareToIgnoreCase("-c2") == 0) {// Localized CREST result
		    ii++;
		    CRESTFile = args[ii++];
		} else if (args[ii].compareToIgnoreCase("-ai") == 0) {// LOH signal file
		    ii++;
		    aiFile = args[ii++];
		    readAI = true;
		} else if (args[ii].compareToIgnoreCase("-lr") == 0) {// LengthRatio power
		    ii++;
		    double d = Double.parseDouble(args[ii++]);
		    if (d > 1) lpower = d;
		} else if (args[ii].compareToIgnoreCase("-gm") == 0) {// GMean power
		    ii++;
		    double d = Double.parseDouble(args[ii++]);
		    if (d > 1) gpower = d;
		} else if (args[ii].compareToIgnoreCase("-svm") == 0) {// Default Value for no SV support
		    ii++;
		    SV_minVal = Double.parseDouble(args[ii++]);
		} else if (args[ii].compareToIgnoreCase("-male") == 0) { //Male
		    male = true;
		    ii++;
		} else if (args[ii].compareToIgnoreCase("-tm") == 0) { //Merge threshold
		    ii++;
		    double d = Double.parseDouble(args[ii++]);
		    if (d >= 0 && d < 1) thresh_merging = d;
		} else if (args[ii].compareToIgnoreCase("-td") == 0) { //Threshold for difference
		    ii++;
		    double d = Double.parseDouble(args[ii++]);
		    if (d >= 0 && d < 1) thresh_diff = d;
		} else if (args[ii].compareToIgnoreCase("-tl") == 0) { //Threshold for log2ratio
		    ii++;
		    double d = Double.parseDouble(args[ii++]);
		    if (d >= 0 && d < 1) thresh_log = d;
		} else if (args[ii].compareToIgnoreCase("-w") == 0) { //Window size
		    ii++;
		    int d = Integer.parseInt(args[ii++]);
		    if (d >= 0) window = d;
		}
	    }
	double distance_thresh = 5 * window;
	double[][] svScores = new double[3][3];
	svScores[2][2] = 1; svScores[0][0] = SV_minVal;
	svScores[0][1] = 0.375 * (1 - SV_minVal) + SV_minVal; svScores[1][0] = svScores[0][1];
	svScores[0][2] = 0.5 + 0.5 * SV_minVal; svScores[2][0] = svScores[0][2];
	svScores[1][2] = 0.875 * (1 - SV_minVal) + SV_minVal; svScores[2][1] = svScores[1][2];
//	svScores[1][2] = 1; svScores[2][1] = svScores[1][2];
	svScores[1][1] = 0.75 * (1 - SV_minVal) + SV_minVal;
	if (CRESTInput == null) noSV = true;
	if (segFile == null) {
	    System.err.println("No CONSERTING output files.");
	    System.exit(-1);
	}
	try {
	    if (readAI) {
		System.out.println("Loading AI input: " + aiFile);
		BufferedReader br = new BufferedReader(new FileReader(aiFile));
		String line = br.readLine();
		String currentChr = null;
		int count = 0, chr = -1;
		int[] itmp = null;
		while ((line = br.readLine()) != null) {
		    StringTokenizer st = new StringTokenizer(line, "\t");
		    String schr = st.nextToken();
		    if (currentChr == null) {
			itmp = new int[1000];
			chr = Integer.parseInt(schr);
			currentChr = schr;
		    } else if (currentChr.compareTo(schr) != 0) {
			if (chr < 24) {
			    snps[chr - 1] = new int[count];
			    System.arraycopy(itmp, 0, snps[chr - 1], 0, count);
//			    System.out.println("Chromosomes " + chr + ":\t" + count + " SNPs");
			}
			count = 0;
			currentChr = schr;
			try {
			    chr = Integer.parseInt(schr);
			} catch (NumberFormatException nfe) {
			    chr = (schr.compareTo("X") == 0 || schr.compareTo("chrX") == 0) ? 23 : 24;
			}
		    }
		    if (chr > 23) continue;
		    if (count == itmp.length) {	
			int[] iitmp = new int[2 * count];
			System.arraycopy(itmp, 0, iitmp, 0, count);
			itmp = iitmp;
		    }
		    itmp[count++] = Integer.parseInt(st.nextToken());
		}
		if (chr <= 22) {
		    snps[chr - 1] = new int[count];
		    System.arraycopy(itmp, 0, snps[chr - 1], 0, count);
		}
		br.close();
	    }
	} catch (IOException ie) {
	    readAI = false;
	}
	try {
	    if (!noSV) {
		System.out.println("Loading original CREST input: " + CRESTInput);
		BufferedReader br = new BufferedReader(new FileReader(CRESTInput));
		String line = null;
		while ((line = br.readLine()) != null) {
		    StringTokenizer st = new StringTokenizer(line, "\t");
		    boolean readCounts = (st.countTokens() > 6);
		    String stmp = st.nextToken();
		    int chrA = 0;
		    try {
			chrA = Integer.parseInt(stmp);
		    } catch (NumberFormatException nfe) {
			if (stmp.compareTo("X") == 0) chrA = 23;
			if (stmp.compareTo("Y") == 0) chrA = 24;
		    }
		    int posA = Integer.parseInt(st.nextToken());
		    boolean oriA = st.nextToken().startsWith("+");
		    int countA = 2, countB = 2;
		    if (readCounts) countA = Integer.parseInt(st.nextToken());
		    stmp = st.nextToken();
		    int chrB = 0;
		    try {
			chrB = Integer.parseInt(stmp);
		    } catch (NumberFormatException nfe) {
			if (stmp.compareTo("X") == 0) chrB = 23;
			if (stmp.compareTo("Y") == 0) chrB = 24;
		    }
		    int posB = Integer.parseInt(st.nextToken());
		    boolean oriB = st.nextToken().startsWith("+");
		    if (readCounts) countB = Integer.parseInt(st.nextToken());
		    SV s = new SV(chrA, posA, oriA, countA, chrB, posB, oriB, countB);
		    if (counts[chrA] == svs[chrA].length) {
			SV[] svtmp = new SV[2 * counts[chrA]];
			System.arraycopy(svs[chrA], 0, svtmp, 0, counts[chrA]);
			svs[chrA] = svtmp;
		    }
		    svs[chrA][counts[chrA]++] = s;
		    if (counts[chrB] == svs[chrB].length) {
			SV[] svtmp = new SV[2 * counts[chrB]];
			System.arraycopy(svs[chrB], 0, svtmp, 0, counts[chrB]);
			svs[chrB] = svtmp;
		    }
		    svs[chrB][counts[chrB]++] = s;
		}
		br.close();
		System.out.println("Loading local CREST input: " + CRESTFile);
		try {
		    br = new BufferedReader(new FileReader(CRESTFile));
		    while ((line = br.readLine()) != null) {
			StringTokenizer st = new StringTokenizer(line, "\t");
			String stmp = st.nextToken();
			int chrA = 0;
			try {
			    chrA = Integer.parseInt(stmp);
			} catch (NumberFormatException nfe) {
			    if (stmp.compareTo("X") == 0) chrA = 23;
			    if (stmp.compareTo("Y") == 0) chrA = 24;
			}
			int posA = Integer.parseInt(st.nextToken());
			boolean oriA = st.nextToken().startsWith("+");
			int countA = Integer.parseInt(st.nextToken());
			stmp = st.nextToken();
			int chrB = 0;
			try {
			    chrB = Integer.parseInt(stmp);
			} catch (NumberFormatException nfe) {
			    if (stmp.compareTo("X") == 0) chrB = 23;
			    if (stmp.compareTo("Y") == 0) chrB = 24;
			}
			int posB = Integer.parseInt(st.nextToken());
			boolean oriB = st.nextToken().startsWith("+");
			int countB = Integer.parseInt(st.nextToken());
			SV s = new SV(chrA, posA, oriA, countA, chrB, posB, oriB, countB);
			boolean match = false;
			for (int i = 0; i < counts[chrA]; i++) {
			    if (s.isSame(svs[chrA][i])) {
				if (s.oriA == svs[chrA][i].oriA) {
				    if (s.countA > svs[chrA][i].countA) svs[chrA][i].countA = s.countA;
				    if (s.countB > svs[chrA][i].countB) svs[chrA][i].countB = s.countB;
				} else {
				    if (s.countA > svs[chrA][i].countB) svs[chrA][i].countB = s.countA;
				    if (s.countB > svs[chrA][i].countA) svs[chrA][i].countA = s.countB;
				}
				match = true;
				break;
			    }
			}
			if (match) continue;
			if (counts[chrA] == svs[chrA].length) {
			    SV[] svtmp = new SV[2 * counts[chrA]];
			    System.arraycopy(svs[chrA], 0, svtmp, 0, counts[chrA]);
			    svs[chrA] = svtmp;
			}
			svs[chrA][counts[chrA]++] = s;
			if (counts[chrB] == svs[chrB].length) {
			    SV[] svtmp = new SV[2 * counts[chrB]];
			    System.arraycopy(svs[chrB], 0, svtmp, 0, counts[chrB]);
			    svs[chrB] = svtmp;
			}
			svs[chrB][counts[chrB]++] = s;
		    }
		    br.close();
		} catch (FileNotFoundException fnfe) {
		    System.err.println("No local CREST output found.");
		}
	    }
	    BufferedReader br = new BufferedReader(new FileReader(segFile));
	    String line = br.readLine();
	    PrintWriter pw = new PrintWriter(new FileWriter(segFile + ".QualityMerge"));
	    pw.println(line + "\tQualityScore");
	    double currentLscore = 0, currentGscore = 0, lscore = 0, gscore = 0, svscore = 1, currentVscore = 0;
	    int[][] segI = new int[200][4];
	    double[][] segD = new double[segI.length][7];
	    int count = 0;
	    while ((line = br.readLine()) != null) {
		StringTokenizer st = new StringTokenizer(line, "\t");
		int chr = Integer.parseInt(st.nextToken()), posA = Integer.parseInt(st.nextToken());
		int posB = 0;
		String ss = st.nextToken();
		int num = Integer.parseInt(st.nextToken());
		try {
		    posB = Integer.parseInt(ss);
		} catch (NumberFormatException nfe) {
		    posB = Math.round(Float.parseFloat(ss));
		}
		double lr = Double.parseDouble(st.nextToken());
		if (lr > 0.9 || (num > 10000 && lr > 0.5) || num > 100000) {
		    lscore = 1;
		} else {
		    lscore = Math.pow(lr/0.9, lpower);
		}
		double diff = Double.parseDouble(st.nextToken());
		double gmean = Double.parseDouble(st.nextToken());
		double dmean = Double.parseDouble(st.nextToken());
		double logratio = Double.parseDouble(st.nextToken());
		double vscore = ((Math.abs(diff) < thresh_diff && Math.abs(logratio) < thresh_log) || (diff * logratio > 0)) ? 1 : 0;
		double g = (male && chr > 22) ? (gmean * 2) : gmean;
		gscore = (g > 2) ? (-1) : Math.pow((1 - Math.abs(g - 1))/0.9, gpower);
		if (readAI && g < 0.9 && (chr < 23 || (!male && chr == 23))) {
		    int c1 = Arrays.binarySearch(snps[chr - 1], posA), c2 = Arrays.binarySearch(snps[chr - 1], posB);
		    if (c1 < 0) c1 = -c1 - 1;
		    if (c2 < 0) c2 = -c2 - 1;
		    double g1 = (c2 - c1) * (snps[chr - 1][snps[chr - 1].length - 1] - snps[chr - 1][0]) * 1.0 /(snps[chr - 1].length * (posB - posA + 1));
		    if (g1 > gscore) gscore = g1;
		}
		if (gscore > 1) gscore = 1;
		double score = 0;
		if (count == 0 || chr != segI[count - 1][0]) {
		    score = 1;
		} else {
		    int oldPosA = posA;
		    if (!noSV) {
			double dist_thresh = (num * window / 2 < distance_thresh) ? num * window / 2 : distance_thresh;
			svscore = SV_minVal;
			double internal = 1.5;
			for (int i = 0; i < counts[chr]; i++) {
			    if (svs[chr][i].chrA == chr && Math.abs(svs[chr][i].posA - oldPosA) <= dist_thresh) {
				//svs[chr][i].confirmA = true;
				int c1 = (svs[chr][i].countA > 1) ? 2 : svs[chr][i].countA;
				int c2 = (svs[chr][i].countB > 1) ? 2 : svs[chr][i].countB;
				if (svScores[c1][c2] > svscore) {
				    svscore = svScores[c1][c2];
				}
				if (Math.abs(svs[chr][i].posA - oldPosA) <= window / 2) {
				    double valtmp = (svs[chr][i].countA + 0.5) * (svs[chr][i].countB + 0.5);
				    if (valtmp > internal) { 
					internal = valtmp;
					posA = svs[chr][i].posA;
				    }
				}
			    }
			    if (svs[chr][i].chrB == chr && Math.abs(svs[chr][i].posB - oldPosA) <= dist_thresh) {
				//svs[chr][i].confirmB = true;
				int c1 = (svs[chr][i].countA > 1) ? 2 : svs[chr][i].countA;
				int c2 = (svs[chr][i].countB > 1) ? 2 : svs[chr][i].countB;
				if (svScores[c1][c2] > svscore) {
				    svscore = svScores[c1][c2];
				}
				if (Math.abs(svs[chr][i].posB - oldPosA) <= window / 2) {
				    double valtmp = (svs[chr][i].countA + 0.5) * (svs[chr][i].countB + 0.5);
				    if (valtmp > internal) { 
					internal = valtmp;
					posA = svs[chr][i].posB;
				    }
				}
			    }
			}
		    }
		    double dscore = Math.abs(diff - segD[count - 1][1]) / thresh_diff;
		    if (dscore > 8) {
			dscore = 2;
		    } else if (dscore > 1) {
			dscore = 1 + (dscore - 1) / 7;
		    }
		    double lrscore = Math.abs(logratio - segD[count - 1][4]) / thresh_log;
		    if (lrscore > 6) {
			lrscore = 2;
		    } else if (lrscore > 1) {
			lrscore = 1 + (lrscore - 1) / 5;
		    }
		    score = svscore * lscore * gscore * currentLscore * currentGscore * vscore * currentVscore * lrscore * dscore;
		    if (svscore >=  svScores[1][1] && score < svscore * lrscore * dscore / 2) score = svscore * lrscore * dscore / 2;
		    if (oldPosA - segI[count - 1][2] < window) {
			segI[count - 1][2] = posA - 1;
		    }
		    //pw.println("\t" + score);
		}
		if (count == segI.length) {
		    int[][] itmp = new int[2 * count][segI[0].length];
		    System.arraycopy(segI, 0, itmp, 0, count);
		    segI = itmp;
		    double[][] dtmp = new double[2 * count][segD[0].length];
		    System.arraycopy(segD, 0, dtmp, 0, count);
		    segD = dtmp;
		}
		segI[count][0] =chr;
		segI[count][1] = posA;
		segI[count][2] = posB;
		segI[count][3] = num;
		segD[count][0] = lr;
		segD[count][1] = diff;
		segD[count][2] = gmean;
		segD[count][3] = dmean;
		segD[count][4] = logratio;
		segD[count][5] = score;
		segD[count][6] = svscore;
		if (score > 0) {
		    currentLscore = lscore;
		    currentGscore = gscore;
		    currentVscore = vscore;
		}
		count++;
	    }
	    br.close();
//	    System.out.println("Entries read: " + count);
	    for (int i = count; i < segD.length; i++) segD[i] = null;
	    while (true) {
		double min_score = 1, min_diff = 1;
		int min_idx = -1;
		for (int i = 0; i < count; i++) {
		    if (segD[i][5] < min_score) {
			min_score = segD[i][5];
			min_diff = Math.abs(segD[i][1] - segD[i - 1][1]);
			min_idx = i;
		    } else if (i > 0 && segD[i][5] == min_score && Math.abs(segD[i][1] - segD[i - 1][1]) < min_diff) {
			min_diff = Math.abs(segD[i][1] - segD[i - 1][1]);
			min_idx = i;
		    }
		}
//		System.out.print(min_idx + " " + min_diff + " " + min_score + " " + count + "\t");
		if (min_score < thresh_merging) {
		    segD[min_idx - 1][1] = (segD[min_idx - 1][1] * segI[min_idx - 1][3] + segD[min_idx][1] * segI[min_idx][3]) / (segI[min_idx - 1][3] + segI[min_idx][3]);
		    segD[min_idx - 1][2] = (segD[min_idx - 1][2] * segI[min_idx - 1][3] + segD[min_idx][2] * segI[min_idx][3]) / (segI[min_idx - 1][3] + segI[min_idx][3]);
		    segD[min_idx - 1][3] = (segD[min_idx - 1][3] * segI[min_idx - 1][3] + segD[min_idx][3] * segI[min_idx][3]) / (segI[min_idx - 1][3] + segI[min_idx][3]);
		    segD[min_idx - 1][4] = (segD[min_idx - 1][4] * segI[min_idx - 1][3] + segD[min_idx][4] * segI[min_idx][3]) / (segI[min_idx - 1][3] + segI[min_idx][3]);
		    segI[min_idx - 1][2] = segI[min_idx][2];
		    segI[min_idx - 1][3] += segI[min_idx][3];
		    segD[min_idx - 1][0] = (segI[min_idx - 1][3] * 100.0) / (segI[min_idx - 1][2] - segI[min_idx - 1][1] + 1);
		    if (min_idx < count - 1) {
//			System.out.println("Removed Entries: " + segI[min_idx][0] + " " + segI[min_idx][1] + " " + segI[min_idx][2]);
			System.arraycopy(segI, min_idx + 1, segI, min_idx, count - min_idx - 1);
			System.arraycopy(segD, min_idx + 1, segD, min_idx, count - min_idx - 1);
		    }
		    segI[count - 1] = null;
		    segD[count - 1] = null;
		    count--;
		    if (min_idx == 1 || segI[min_idx - 2][0] != segI[min_idx - 1][0]) {
			segD[min_idx - 1][5] = 1;
		    } else {
			double g = (segI[min_idx - 2][0] > 22 && male) ? segD[min_idx - 2][2] * 2 : segD[min_idx - 2][2];
			int idx = min_idx - 2;
			while (g > 2 && idx >= 0 && segI[idx][0] == segI[min_idx - 1][0]) {
			    idx--;
			    g = (segI[idx][0] > 22 && male) ? segD[idx][2] * 2 : segD[idx][2];
			}
			double gscore1 = (g > 2) ? -1 : Math.pow((1 - Math.abs(g - 1))/0.9, gpower);
			if (readAI && g < 0.9 && segI[idx][0] < 23) {
			    int chr = segI[idx][0], posA = segI[idx][1], posB = segI[idx][2];
			    int c1 = Arrays.binarySearch(snps[chr - 1], posA), c2 = Arrays.binarySearch(snps[chr - 1], posB);
			    if (c1 < 0) c1 = -c1 - 1;
			    if (c2 < 0) c2 = -c2 - 1;
			    double g1 = (c2 - c1) * (snps[chr - 1][snps[chr - 1].length - 1] - snps[chr - 1][0]) * 1.0 /(snps[chr - 1].length * (posB - posA + 1));
			    if (g1 > gscore1) gscore1 = g1;
			}
			if (gscore1 > 1) gscore1 = 1;
			double lscore1 = (segD[idx][0] > 0.9 || (segI[idx][3] > 10000 && segD[idx][0] > 0.5) || segI[idx][3] > 100000) ? 1 : Math.pow(segD[idx][0]/0.9, lpower);
			double lscore2 = (segD[min_idx - 1][0] > 0.9 || (segI[min_idx - 1][3] > 10000 && segD[min_idx - 1][0] > 0.5) || segI[min_idx - 1][3] > 100000) ? 1 : Math.pow(segD[min_idx - 1][0]/0.9, lpower);
			double vscore1 = ((Math.abs(segD[idx][1]) < thresh_diff && Math.abs(segD[idx][4]) < thresh_log) || segD[idx][1] * segD[idx][4] > 0) ? 1 : 0;
			double vscore2 = ((segD[min_idx - 1][1] < thresh_diff && segD[min_idx - 1][4] < thresh_log) || segD[min_idx - 1][1] * segD[min_idx -1][4] > 0) ? 1 : 0;
			g = (segI[min_idx - 1][0] > 22 && male) ? segD[min_idx - 1][2] * 2 : segD[min_idx - 1][2];
			double gscore2 = (g > 2) ? 0 : Math.pow((1 - Math.abs(g - 1))/0.9, gpower);
			if (readAI && g < 0.9 && (segI[min_idx - 1][0] < 23 || (!male && segI[min_idx - 1][0] == 23))) {
			    int chr = segI[min_idx - 1][0], posA = segI[min_idx - 1][1], posB = segI[min_idx - 1][2];
			    int c1 = Arrays.binarySearch(snps[chr - 1], posA), c2 = Arrays.binarySearch(snps[chr - 1], posB);
			    if (c1 < 0) c1 = -c1 - 1;
			    if (c2 < 0) c2 = -c2 - 1;
			    double g1 = (c2 - c1) * (snps[chr - 1][snps[chr - 1].length - 1] - snps[chr - 1][0]) * 1.0 /(snps[chr - 1].length * (posB - posA + 1));
			    if (g1 > gscore2) gscore2 = g1;
			}
			if (gscore2 > 1) gscore2 = 1;
			double dscore = Math.abs(segD[min_idx - 1][1] - segD[idx][1]) / thresh_diff;
			if (dscore > 8) {
			    dscore = 2;
			} else if (dscore > 1) {
			    dscore = 1 + (dscore - 1) / 7;
			}
			double lrscore = Math.abs(segD[min_idx - 1][4] - segD[idx][4]) / thresh_log;
			if (lrscore > 6) {
			    lrscore = 2;
			} else if (lrscore > 1) {
			    lrscore = 1 + (lrscore - 1) / 5;
			}
			segD[min_idx - 1][5] = segD[min_idx - 1][6] * lscore1 * gscore1 * lscore2 * gscore2 * vscore1 * vscore2 * lrscore * dscore;
			if (segD[min_idx - 1][6] >= svScores[1][1] &&  segD[min_idx - 1][5] < segD[min_idx - 1][6] * lrscore * dscore / 2) segD[min_idx - 1][5] = segD[min_idx - 1][6] * lrscore * dscore / 2;
//			if (segI[min_idx - 1][1] - segI[min_idx - 2][2] > 100) segD[min_idx - 1][5] /= 4;
		    }
		    if (min_idx > count  - 1) continue;
		    if (segI[min_idx - 1][0] != segI[min_idx][0]) {
			segD[min_idx][5] = 1;
		    } else {
			double g = (segI[min_idx][0] > 22 && male) ? segD[min_idx][2] * 2 : segD[min_idx][2];
			double gscore1 = (g > 2) ? 0 : Math.pow((1 - Math.abs(g - 1))/0.9, gpower);
			if (readAI && g < 0.9 && (segI[min_idx][0] < 23 || (!male && segI[min_idx][0] == 23))) {
			    int chr = segI[min_idx][0], posA = segI[min_idx][1], posB = segI[min_idx][2];
			    int c1 = Arrays.binarySearch(snps[chr - 1], posA), c2 = Arrays.binarySearch(snps[chr - 1], posB);
			    if (c1 < 0) c1 = -c1 - 1;
			    if (c2 < 0) c2 = -c2 - 1;
			    double g1 = (c2 - c1) * (snps[chr - 1][snps[chr - 1].length - 1] - snps[chr - 1][0]) * 1.0 /(snps[chr - 1].length * (posB - posA + 1));
			    if (g1 > gscore1) gscore1 = g1;
			}
			if (gscore1 > 1) gscore1 = 1;
			g = (segI[min_idx - 1][0] > 22 && male) ? segD[min_idx - 1][2] * 2 : segD[min_idx - 1][2];
			int idx = min_idx - 1;
			while (g > 2 && idx >= 0 && segI[idx][0] == segI[min_idx][0]) {
			    idx--;
			    g = (segI[idx][0] > 22 && male) ? segD[idx][2] * 2 : segD[idx][2];
			}
			double gscore2 = (g > 2) ? -1 : Math.pow((1 - Math.abs(g - 1))/0.9, gpower);
			if (readAI && g < 0.9 && segI[idx][0] < 23) {
			    int chr = segI[idx][0], posA = segI[idx][1], posB = segI[idx][2];
			    int c1 = Arrays.binarySearch(snps[chr - 1], posA), c2 = Arrays.binarySearch(snps[chr - 1], posB);
			    if (c1 < 0) c1 = -c1 - 1;
			    if (c2 < 0) c2 = -c2 - 1;
			    double g1 = (c2 - c1) * (snps[chr - 1][snps[chr - 1].length - 1] - snps[chr - 1][0]) * 1.0 /(snps[chr - 1].length * (posB - posA + 1));
			    if (g1 > gscore2) gscore2 = g1;
			}
			if (gscore2 > 1) gscore2 = 1;
			double lscore1 = (segD[min_idx][0] > 0.9 || (segI[min_idx][3] > 10000 && segD[min_idx][0] > 0.5) || segI[min_idx][3] > 100000) ? 1 : Math.pow(segD[min_idx][0]/0.9, lpower);
			double lscore2 = (segD[idx][0] > 0.9 || (segI[idx][3] > 10000 && segD[idx][0] > 0.5) || segI[idx][3] > 100000) ? 1 : Math.pow(segD[idx][0]/0.9, lpower);
			double vscore1 = ((Math.abs(segD[min_idx][1]) < thresh_diff && Math.abs(segD[min_idx][4]) < thresh_log) || segD[min_idx][1] * segD[min_idx][4] > 0) ? 1 : 0;
			double vscore2 = ((segD[idx][1] < thresh_diff && segD[idx][4] < thresh_log) || segD[idx][1] * segD[idx][4] > 0) ? 1 : 0;
			double dscore = Math.abs(segD[idx][1] - segD[min_idx][1]) / thresh_diff;
			if (dscore > 8) {
			    dscore = 2;
			} else if (dscore > 1) {
			    dscore = 1 + (dscore - 1) / 7;
			}
			double lrscore = Math.abs(segD[idx][4] - segD[min_idx][4]) / thresh_log;
			if (lrscore > 6) {
			    lrscore = 2;
			} else if (lrscore > 1) {
			    lrscore = 1 + (lrscore - 1) / 5;
			}
			segD[min_idx][5] = segD[min_idx][6] * lscore1 * gscore1 * lscore2 * gscore2 * vscore1 * vscore2 * lrscore * dscore;
			if (segD[min_idx][6] >= svScores[1][1] &&  segD[min_idx][5] < segD[min_idx][6] * lrscore * dscore / 2) segD[min_idx][5] = segD[min_idx][6] * lrscore * dscore / 2;
		    }

		} else {
		    break;
		}
	    }
	    NumberFormat formatter = new DecimalFormat("0.000");
	    for (int i = 0; i < count; i++) {
		for (int j = 0; j < 4; j++) pw.print(segI[i][j] + "\t");
		for (int j = 0; j < 5; j++) pw.print(formatter.format(segD[i][j]) + "\t");
		pw.println(formatter.format(segD[i][5]));
	    }
	    pw.close();
	    if (!noSV) {
		pw = new PrintWriter(new FileWriter(segFile + ".CNAcalls"));
		pw.println("chrom\tloc.start\tloc.end\tnum.mark\tLog2Ratio");
		for (int i = 0; i < count; i++) {
		    if (Math.abs(segD[i][1]) >= thresh_diff && Math.abs(segD[i][4]) >= thresh_log) {
			if ((male && segI[i][0] > 22 && Math.abs(segD[i][2] - 0.5) < 0.275) || ((segI[i][0] <= 22 || ((!male) && segI[i][0] == 23)) && Math.abs(segD[i][2] - 1) < 0.55)) {
			    if (segI[i][0] < 23) {
				pw.print(segI[i][0]);
			    } else if (segI[i][0] == 23) {
				pw.print("X");
			    } else {
				pw.print("Y");
			    }
			    for (int j = 1; j < 4; j++) pw.print("\t" + segI[i][j]);
			    pw.println("\t" + formatter.format(segD[i][4]));
			}
		    }
		    double dist_thresh = (segI[i][3] * window / 2 < distance_thresh) ? segI[i][3] * window / 2 : distance_thresh;
			for (int j = 0; j < counts[segI[i][0]]; j++) {
			    if (svs[segI[i][0]][j].chrA == segI[i][0] && Math.abs(svs[segI[i][0]][j].posA - segI[i][1]) <= dist_thresh) {
				svs[segI[i][0]][j].confirmA = true;
			    }
			    if (svs[segI[i][0]][j].chrB == segI[i][0] && Math.abs(svs[segI[i][0]][j].posB - segI[i][1]) <= dist_thresh) {
				svs[segI[i][0]][j].confirmB = true;
			    }
			}
		}
		pw.close();
		System.out.println("Writing local CREST file with quality: " + CRESTFile + ".QualityMerge");
		pw = new PrintWriter(new FileWriter(CRESTFile + ".QualityMerge"));
		for (int i = 1; i < 25; i++) {
		    for (int j = 0; j < counts[i]; j++) {
			if (!svs[i][j].checked) {
			    if (svs[i][j].confirmA || svs[i][j].confirmB) pw.println(svs[i][j]);
			    svs[i][j].checked = true;
			}
		    }
		}
		pw.close();
	    }
	} catch (IOException e) {
	    e.printStackTrace();
	    System.exit(-1);
	}
		    
    }



    class SV {
	int chrA, posA, chrB, posB, countA=2, countB=2;
	boolean oriA, oriB, confirmA = false, confirmB = false, checked = false;

	SV(int cA, int pA, boolean oA, int cB, int pB, boolean oB){
	    chrA = cA; posA = pA; oriA = oA; chrB = cB; posB = pB; oriB = oB;
	}
	SV(int cA, int pA, boolean oA, int countA, int cB, int pB, boolean oB, int countB){
	    this(cA, pA, oA, cB, pB, oB);
	    this.countA = countA; this.countB = countB;
	}

	boolean isSame(SV s) {
	    if ((s.chrA == chrA) && (s.posA == posA) && (s.oriA == oriA) && (s.chrB == chrB) && (s.posB == posB) && (s.oriB == oriB)) return true;
	    if ((s.chrA == chrB) && (s.posA == posB) && (s.oriA != oriA) && (s.chrB == chrA) && (s.posB == posA) && (s.oriB != oriA)) return true;
	    return false;
	}
	public String toString() {
	    String res = "" + chrA;
	    if (chrA == 0) res = "NonStandard";
	    if (chrA == 23) res = "X";
	    if (chrA == 24) res = "Y";
	    res += "\t" + posA + (oriA ? "\t+\t" : "\t-\t") + countA + "\t";
	    if (chrB == 0) {
		res += "NonStandard";
	    } else if (chrB == 23) {
		res += "X";
	    } else if (chrB == 24) {
		res += "Y";
	    } else {
		res += chrB;
	    }
	    res += "\t" + posB + (oriB ? "\t+\t" : "\t-\t") + countB;
	    return res;
	}
    }



}
